-- Modul JOCS D'ORDINADOR (games)
-- Programadors: Sergio Blanco Cuaresma    (E.T.I. Gesti)
--               Francesc Galiano Baixauli (E.T.I. Gesti)
-- Data: Mar 2001
--
-- Aquest modul conte les funcions necesaries per manipular una base
-- de dades de jocs. Inicialitza, busca, afegeix jocs, canvia records,
-- afegeix partides, llista...
-- *****************************************************************

with Ada.Text_IO; use Ada.Text_IO;
package body games is
	package enter_io is new ada.text_io.integer_io(integer); use enter_io;
	package boolea_io is new ada.text_io.enumeration_io(boolean); use boolea_io;	

      -- Acci per inicialitzar la base de dades
   	procedure init(jocs: in out basedades) is
		begin
			jocs.total:=0;
		end init;
	
      -- Acci per afegir un joc a la base de dades
	procedure afegir(jocs: in out basedades) is
		lnom: natural;
		
		begin
			new_line;
   
                  -- Demanem el nom del joc i omplim amb blancs la resta de l'string
			put("Nom del joc: ");
			get_line(jocs.fitxer(jocs.total+1).nom, lnom);
			for i in lnom+1..jocs.fitxer(jocs.total+1).nom'last loop
				jocs.fitxer(jocs.total+1).nom(i):=' ';
			end loop;
			
			put_line("Data d'adquisicio...");
			put("  Dia (1..31) : ");
			get(jocs.fitxer(jocs.total+1).data.dia);
			put("  Mes (1..12): ");
			get(jocs.fitxer(jocs.total+1).data.mes);
			put("  Any: ");
			get(jocs.fitxer(jocs.total+1).data.any);
			
			put("Preu: ");
			get(jocs.fitxer(jocs.total+1).preu);
			put("Partides jugades: ");
			get(jocs.fitxer(jocs.total+1).partides);
			put("Puntuacio maxima (record): ");
			get(jocs.fitxer(jocs.total+1).maxpunts);

			jocs.total:=jocs.total+1;
			new_line;

		    exception
				when data_error => put_line(">> Valor incorrecte!!");
				when constraint_error => put_line(">> Valor fora de rang!!");

		end afegir;

	-- Acci per afegir partides a un joc de la base de dades
      procedure afegir_partides(jocs: in out basedades; nom: in string; partides: in natural) is
		bucle: positive;
		begin
			bucle:=1;
                  -- Busquem el joc
			while (bucle<jocs.total) and (jocs.fitxer(bucle).nom/=nom) loop
				bucle:=bucle+1;
			end loop;
   
			if (jocs.fitxer(bucle).nom=nom) then
				jocs.fitxer(bucle).partides:=jocs.fitxer(bucle).partides + partides;
			elsif (jocs.fitxer(bucle).nom/=nom) then
				put_line("Joc no trobat.");
			end if;

		    exception
				when data_error => put_line(">> Numero resultant massa gran!!");

		end afegir_partides;


      -- Acci per canviar el record d'un joc de la base de dades
	procedure nou_record(jocs: in out basedades; nom: in string; nrecord: in natural) is
		bucle: positive;
		begin
			bucle:=1;
			-- Busquem el joc
                  while (bucle<jocs.total) and (jocs.fitxer(bucle).nom/=nom) loop
				bucle:=bucle+1;
			end loop;

                  -- Abans de canviar el record s'assegura que el nou es superior a l'antic
			if (jocs.fitxer(bucle).nom=nom) and (nrecord>jocs.fitxer(bucle).maxpunts) then
				jocs.fitxer(bucle).maxpunts:=nrecord;
			elsif (jocs.fitxer(bucle).nom=nom) and (nrecord<=jocs.fitxer(bucle).maxpunts) then
				put_line("El record vell es superior al nou.");
			elsif (jocs.fitxer(bucle).nom/=nom) then
				put_line("Joc no trobat.");
			end if;
		end nou_record;

	-- Funci per retornar el total de jocs que conte la base de dades
      function totaljocs(jocs: basedades) return integer is
		begin
			return jocs.total;
		end totaljocs;

      -- Funci que retorna verdader si es troba un joc donat a la base de dades
	function trobar(jocs: basedades; nom: string) return boolean is
		bucle: positive;
		begin
			bucle:=1;
                  -- Busquem el joc
			while (bucle<jocs.total) and (jocs.fitxer(bucle).nom/=nom) loop
				bucle:=bucle+1;
			end loop;
			return (jocs.fitxer(bucle).nom=nom);
		end trobar;


      -- Acci per llistar tots els titols dels jocs de la base de dades
	procedure llistar(jocs: in basedades) is
		begin
			put_line("Procesant...");
			for bucle in 1..jocs.total loop
				put("(");put(bucle, width=>3);put(") "); put(jocs.fitxer(bucle).nom);new_line;
			end loop;
			put_line("Fi.");
		end llistar;


      -- Acci per mostrar tota la informacio d'un joc
	procedure mostrar(jocs: in basedades; nom: in string) is
		bucle: positive;
		begin
			bucle:=1;
                  -- Busquem el joc
			while (bucle<jocs.total) and (jocs.fitxer(bucle).nom/=nom) loop
				bucle:=bucle+1;
			end loop;

			if (jocs.fitxer(bucle).nom=nom) then
				new_line;
				put_line("----");
				put("Nom: ");put_line(jocs.fitxer(bucle).nom);
				put("Data: ");
				put(jocs.fitxer(bucle).data.dia, width=>2);put("/");
				put(jocs.fitxer(bucle).data.mes, width=>2);put("/");
				put(jocs.fitxer(bucle).data.any, width=>4);new_line;
				put("Preu: ");put(jocs.fitxer(bucle).preu, width=>4); new_line;
				put("Partides: ");put(jocs.fitxer(bucle).partides, width=>4); new_line;
				put("Maxima puntuacio: ");put(jocs.fitxer(bucle).maxpunts, width=>4); new_line;
				put_line("----");
			else
				put("Joc no trobat."); new_line;
			end if;
		end mostrar;

      -- Acci per mostrar els titols de tots els jocs amb un minim de partides
      procedure minpartides(jocs: in basedades; partides: in natural) is
		begin
			put_line("Procesant...");
                  -- Fem un recorregut i mostrem els que compleixin la condicio
			for bucle in 1..jocs.total loop
				if (jocs.fitxer(bucle).partides>=partides) then
					put("(");put(bucle, width=>3);put(") "); put(jocs.fitxer(bucle).nom);
					new_line;
				end if;
			end loop;
			put_line("Fi.");
		end minpartides;

	
      -- Acci per mostrar els titols de tots els jocs comprats a partir d'un any
      procedure any(jocs: in basedades; any: natural) is
		begin
			put_line("Procesant...");
                  -- Fem un recorregut i mostrem els que compleixin la condicio
			for bucle in 1..jocs.total loop
				if (jocs.fitxer(bucle).data.any=any) then
					put("(");put(bucle, width=>3);put(") "); put(jocs.fitxer(bucle).nom);
					new_line;
				end if;
			end loop;
			put_line("Fi.");
		end any;


end games;